
-- This addon listens for quest turn-ins of the arcane mana feeding quests. The
-- turn-in time is stored for each corresponding NPC. The remaining time until
-- the quests can be turned in again is computed from this, and presented to
-- the user.



-- The arcane thirst level for each NPC will be polled at a fixed interval,
-- rather than every update of the user interface.
local UPDATE_INTERVAL = 1

-- The NPC's paired with their corresponding faction and quest ID's.
local FRIENDS = {
    {
        name = "Oculeth",
        factionID = 1862,
        questID = 43106,
    },
    {
        name = "Thalyssra",
        factionID = 1860,
        questID = 43994,
    },
    {
        name = "Valtrois",
        factionID = 1919,
        questID = 43995,
    },
}

-- Constructs and returns a message indicating when the NPC's feeding quest
-- will again be available.
local function _GetThirstMessage(state)
    local MESSAGE_LATER = "|cff66ff66%s|r";
    local MESSAGE_SOON = "|cffffff66%s|r";
    local MESSAGE_NOW = "|cffff6666Now|r";
    local MESSAGE_UNKNOWN = "|cff6666ffUnknown|r";

    if state.turnInTime then
        local timeLeft = (state.turnInTime + 86400) - time();
        if timeLeft <= 0 then
            return MESSAGE_NOW;
        end

        if timeLeft <= 3600 then
            return MESSAGE_SOON.format(SecondsToTime(timeLeft));
        else
            return MESSAGE_LATER.format(SecondsToTime(timeLeft));
        end
    end

    return MESSAGE_UNKNOWN;
end

-- Retrieves the state table for the given NPC.
local function _GetState(factionID)
    if not ARCANE_THIRST_STATE then
        ARCANE_THIRST_STATE = {};
    end

    if not ARCANE_THIRST_STATE[factionID] then
        ARCANE_THIRST_STATE[factionID] = {};
    end

    return ARCANE_THIRST_STATE[factionID];
end

-- Updates the text in the frame to reflect the most recent state collected
-- for each NPC.
local function _UpdateFrame()
    for i = 1, #FRIENDS do
        local friend = FRIENDS[i];
        local state = _GetState(friend.factionID);
        local level_text = _GetThirstMessage(state);
        _G["ArcaneThirstFriend" .. i .. "Name"]:SetText(friend.name);
        _G["ArcaneThirstFriend" .. i .. "Level"]:SetText(level_text);
    end
end

-- Updates the quest availability time for all NPC's.
local function _UpdateState()
    for i = 1, #FRIENDS do
        local friend = FRIENDS[i];
        local state = _GetState(friend.factionID);

        local _, level, _, _, _, _, levelText, threshold, nextThreshold =
            GetFriendshipReputation(friend.factionID)
        local newLevel = level - threshold;
        local levelRange = nextThreshold - threshold;

        -- The transition from one arcane thirst level to another can be used
        -- to compute a reasonable estimate of the quest turn-in time. The
        -- minimum possible arcane thirst level indicates that the quest should
        -- be available.

        if newLevel <= 1 then
            state.turnInTime = time() - 86400;
        elseif (state.turnInTime == nil) and
               (state.level ~= nil) and
               (state.level ~= newLevel) then
           local nextTurnInTime = (86400 * newLevel) / levelRange;
           local timeSinceTurnIn = 86400 - nextTurnInTime;
           state.turnInTime = time() - timeSinceTurnIn;
        end

        state.level = newLevel;
        state.levelText = levelText;
    end
end

-- Updates the arcane thirst state of all NPC's, and updates the UI frame if
-- the frame is visible.
local function _UpdateStateAndFrame(force)
    _UpdateState();
    if force or ArcaneThirstFrame:IsVisible() then
        _UpdateFrame();
    end
end

-- Stores the time at which the quest was turned in.
local function ArcaneThirstFrame_QUEST_TURNED_IN(questID, experience, money)
    for i = 1, #FRIENDS do
        local friend = FRIENDS[i];
        if friend.questID == questID then
            state = _GetState(friend.factionID)
            state.turnInTime = time();
            _UpdateStateAndFrame();
        end
    end
end

-- Starts the drag operation if the left mouse button was pressed.
function ArcaneThirstFrame_OnClick(self, button, down)
    if button == "RightButton" and down then
        self:Hide();
    end
end

function ArcaneThirstFrame_OnDragStart(self, button)
    if button == "LeftButton" then
        self:StartMoving();
    end
end

function ArcaneThirstFrame_OnDragStop(self)
    self:StopMovingOrSizing();
end

function ArcaneThirstFrame_OnEvent(self, event, ...)
    if event == "QUEST_TURNED_IN" then
        ArcaneThirstFrame_QUEST_TURNED_IN(...);
	end
end

function ArcaneThirstFrame_OnLoad(self)
    self.updateTime = 0;
    self.updateInterval = UPDATE_INTERVAL;

    -- Allow ESCape to close the window.
    tinsert(UISpecialFrames, self:GetName());

    -- Support dragging within the confines of the screen.
    self:RegisterForDrag("LeftButton");
    self:SetClampedToScreen(true);
    self:SetUserPlaced(true);

    self:RegisterEvent("QUEST_TURNED_IN");
end

-- Forces an update of the state, then updates the frame. This prevents the
-- frame from appearing without any information for the friends.
function ArcaneThirstFrame_OnShow(self)
    _UpdateStateAndFrame(true);
end

-- Counts down over the update interval, then updates the stored state. If the
-- frame is visible, the frame is also updated.
function ArcaneThirstFrame_OnUpdate(self, elapsedTime)
    self.updateTime = self.updateTime + elapsedTime;
    if (self.updateTime < self.updateInterval) then
        return
    end
    self.updateTime = 0;

    _UpdateStateAndFrame();
end

-- Registers slash commands to open the frame.
SLASH_ARCANETHIRST1, SLASH_ARCANETHIRST2 = "/arcanethirst", "/thirst"
function SlashCmdList.ARCANETHIRST(message, editBox)
    ArcaneThirstFrame:Show();
end
